"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_utils_1 = require("./path.utils");
const fs_utils_1 = require("./fs.utils");
const path_1 = require("path");
/**
 * Delete files and folders created by a previous build.
 * @internal
 */
function cleanTargets(targets, compilerOptions, basePath = process.cwd()) {
    protectSensitiveFolders(targets, compilerOptions, basePath);
    if (Array.isArray(targets)) {
        for (let target of targets) {
            target = path_utils_1.ensureAbsolutePath(target, basePath);
            console.log('Cleaning:', path_utils_1.relativeToCWD(target));
            fs_utils_1.rmrf(target);
        }
    }
    else {
        const { outDir, outFile, declarationDir } = compilerOptions;
        if (targets.outDir && outDir) {
            console.log('Cleaning outDir:', path_utils_1.relativeToCWD(outDir));
            fs_utils_1.rmrf(outDir);
        }
        if (targets.declarationDir && declarationDir) {
            console.log('Cleaning declarationDir:', path_utils_1.relativeToCWD(declarationDir));
            fs_utils_1.rmrf(declarationDir);
        }
        if (targets.outFile && outFile) {
            console.log('Cleaning outFile:', path_utils_1.relativeToCWD(outFile));
            fs_utils_1.rmrf(outFile);
        }
    }
    // Pause on windows to try to work around eventual lingering file handles
    if (process.platform === 'win32') {
        const until = Date.now() + 250;
        while (Date.now() < until) { }
    }
}
exports.default = cleanTargets;
/**
 * @internal
 */
function protectSensitiveFolders(targets, compilerOptions, basePath) {
    const cwd = process.cwd();
    let protectedDirs = [cwd, ...path_utils_1.parentPaths(cwd)];
    if (cwd !== basePath) {
        protectedDirs.push(basePath, ...path_utils_1.parentPaths(basePath));
    }
    // `compilerOptions` properties returns unix separators in windows paths so we must normalize
    const rootDir = compilerOptions.rootDir ? path_1.normalize(compilerOptions.rootDir) : undefined;
    if (rootDir && rootDir !== cwd && rootDir !== basePath) {
        protectedDirs.push(rootDir, ...path_utils_1.parentPaths(rootDir));
    }
    // Dedupe
    protectedDirs = [...new Set(protectedDirs)];
    const check = (target) => {
        for (const dir of protectedDirs) {
            if (target === dir)
                throw Error(`You cannot delete ${target}`);
        }
    };
    if (Array.isArray(targets)) {
        for (let target of targets) {
            target = path_utils_1.ensureAbsolutePath(target, basePath);
            check(target);
        }
    }
    else {
        for (const target of Object.keys(targets)) {
            if (compilerOptions[target]) {
                // We can have outDir/declarationDir to be the same as rootDir
                check(compilerOptions[target]);
            }
        }
    }
}
