"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
const path_1 = require("path");
const path_utils_1 = require("./path.utils");
const fs_utils_1 = require("./fs.utils");
const log_utils_1 = require("./log.utils");
/**
 * Copy non-typescript files to `outDir`.
 * @param emittedFiles files previously emitted by the compiler and protected from being overwritten
 * @internal
 */
function copyOtherFiles(program, emittedFiles = []) {
    console.log('Copying other files');
    const { outDir, declarationDir, listEmittedFiles } = program.getCompilerOptions();
    if (outDir == null) {
        console.error(log_utils_1.red('Cannot copy: you must define `outDir` in the compiler options'));
        return;
    }
    const srcDir = program.getCommonSourceDirectory();
    if (!srcDir)
        throw Error('Cannot copy: issue with internal typescript method `getCommonSourceDirectory`');
    // Exclude typescript files and outDir/declarationDir if previously emitted in the same folder
    const excludes = ['**/*.ts', outDir];
    if (declarationDir)
        excludes.push(declarationDir);
    const otherFiles = matchAllFilesBut(srcDir, excludes);
    // Track copied files to list them later if needed
    const copiedFiles = [];
    for (const srcFile of otherFiles) {
        const destFile = path_1.resolve(outDir, path_1.relative(srcDir, srcFile));
        // Avoid overwriting precedently emitted files (js or json if resolveJsonModule is true)
        // Need to normalize because emitted files have unix separators in windows paths
        const wasEmitted = emittedFiles.some((emittedFile) => path_1.normalize(emittedFile) === destFile);
        if (wasEmitted) {
            console.warn(`Won't override previously emitted file: ${path_utils_1.relativeToCWD(destFile)}`);
            continue;
        }
        fs_utils_1.cp(srcFile, destFile, wasEmitted);
        copiedFiles.push(destFile);
    }
    if (listEmittedFiles)
        console.log('Copied files:\n' + copiedFiles.join('\n'));
}
exports.default = copyOtherFiles;
/**
 * @internal
 */
function matchAllFilesBut(path, excludes) {
    return typescript_1.sys.readDirectory(path, undefined, excludes, undefined);
}
