"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const path_utils_1 = require("./path.utils");
const log_utils_1 = require("./log.utils");
const clean_addon_1 = __importDefault(require("./clean-addon"));
const copy_addon_1 = __importDefault(require("./copy-addon"));
/**
 * Compile ts files by creating a compilation object using the compiler API and emitting js files.
 * @public
 */
function build(options) {
    const program = createProgramFromConfig(options);
    emit(program, options);
}
exports.build = build;
/**
 * Create a compilation object using the compiler API.
 * @public
 */
function createProgramFromConfig({ basePath, configFilePath, compilerOptions, include, exclude, files, extends: extend, // cuz keyword
references, }) {
    let config = {};
    if (configFilePath) {
        configFilePath = path_utils_1.ensureAbsolutePath(configFilePath, basePath);
        console.log(`Retrieving ${configFilePath}`);
        const readResult = typescript_1.default.readConfigFile(configFilePath, typescript_1.default.sys.readFile);
        if (readResult.error)
            log_utils_1.logDiagnostics([readResult.error], true);
        config = readResult.config;
    }
    config.compilerOptions = Object.assign({}, config.compilerOptions, compilerOptions);
    if (include)
        config.include = include;
    if (exclude)
        config.exclude = exclude;
    if (files)
        config.files = files;
    if (extend)
        config.extends = extend;
    if (references)
        config.references = references;
    const { options, fileNames, projectReferences, errors } = typescript_1.default.parseJsonConfigFileContent(config, typescript_1.default.sys, basePath, undefined, configFilePath);
    log_utils_1.logDiagnostics(errors, true);
    const program = typescript_1.default.createProgram({
        options,
        rootNames: fileNames,
        projectReferences,
    });
    return program;
}
exports.createProgramFromConfig = createProgramFromConfig;
/**
 * Compile Typescript files and emit diagnostics if any.
 * @public
 */
function emit(program, { basePath, clean, copyOtherToOutDir } = {}) {
    const options = program.getCompilerOptions();
    if (clean)
        clean_addon_1.default(clean, options, basePath);
    if (options.listFiles)
        console.log('Files to compile:\n' + program.getRootFileNames().join('\n'));
    console.log('Compilation started');
    const { diagnostics, emitSkipped, emittedFiles } = program.emit();
    if (options.listEmittedFiles && emittedFiles)
        console.log('Emitted files:\n' + emittedFiles.join('\n'));
    // https://github.com/dsherret/ts-morph/issues/384
    const allDiagnostics = typescript_1.default.getPreEmitDiagnostics(program).concat(diagnostics);
    log_utils_1.logDiagnostics(allDiagnostics, options.pretty);
    if (!options.noEmit && emitSkipped) {
        console.error(log_utils_1.red('Compilation failed'));
        return;
    }
    if (copyOtherToOutDir)
        copy_addon_1.default(program, emittedFiles);
    if (allDiagnostics.length)
        console.log(log_utils_1.yellow(`Compilation done with ${allDiagnostics.length} errors`));
    else
        console.log(log_utils_1.green('Compilation successful'));
}
exports.emit = emit;
